/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PIC_CONFIG_H
#define PIC_CONFIG_H

#include <qmap.h>
#include <qstringlist.h>

#include "common/global/global.h"
#include "common/common/misc.h"
#include "pic.h"

namespace Pic
{
class Data;

class Config
{
public:
  class Value {
  public:
    QString name, cname;
    QStringList extraCNames;
    uint value;
    bool operator <(const Value &cv) const { return value<cv.value; }
  };

  class Mask {
  public:
    QString name;
    uint value;
    QValueVector<Value> values; // ordered from lower to higher
    bool operator <(const Mask &cm) const { return value<cm.value; }
  };

  class Word {
  public:
    QString name;
    QStringList ignoredCNames;
    uint wmask, pmask, cmask; // write, protected, and checksum bits masks
    uint bvalue;              // blank value
    QValueVector<Mask> masks; // ordered from lower to higher
    uint usedMask() const;
  };

  enum BlockType { CodeBlock = 0, DataBlock, CalibrationBlock, BootBlock, ConfigBlock, Nb_BlockTypes };
  enum ProtectionType { ProgramProtected = 0, WriteProtected, ReadProtected, Nb_ProtectionTypes };
  QString maskName(ProtectionType ptype, BlockType btype, int block = -1) const;

public:
  Config(const Pic::Data &data) : _data(data) {}
  enum { MAX_NB_BLOCKS = 8 };
  QValueVector<Word> _words;

  const Value *findValue(const QString &mask, const QString &value) const;
  const Mask *findMask(const QString &mask, uint *wordIndex = 0) const;
  static bool hasMaskName(const QString &mask);
  static QString maskLabel(const QString &mask);
  bool checkValueName(const QString &mask, const QString &name) const;
  static QString valueLabel(const QString &mask, const QString &name);

  QString noneProtectedValueName() const;
  bool hasBootBlock() const;
  uint nbBlocks() const;
  Range extractRange(const QString &name, BlockType type) const;

private:
  class MapData {
  public:
    MapData() {}
    MapData(int i, int b) : index(i), block(b) {}
    int index, block;
  };
  static QMap<QString, MapData> &masks();
  static QMap<QString, MapData> *_masks; // mask name -> index in DATA

  struct NameData {
    const char *name, *label;
  };
  enum Type { Fixed, ValueDouble, ValueUInt, Ratio, MemoryRange, Toggle, Pin, Pins };
  class Data {
  public:
    const NameData mask;
    Type type;
    const NameData values[50];
  };
  static const Data DATA[];

private:
  const Pic::Data &_data;

  struct ProtectionData {
    const char *name[Nb_ProtectionTypes];
    MemoryRangeType type;
  };
  static const ProtectionData PROTECTION_DATA[Nb_BlockTypes];
  static bool extractRanges(const QString &name, QValueVector<uint> &starts, uint &end, bool &ok);
  class ProtectedRange {
  public:
    QValueVector<uint> starts, ends;
  };
  ProtectedRange extractRange(const QString &mask, const QString &name, bool &ok) const;
  bool isAllProtectedValueName(const QString &name) const;
  bool isNoneProtectedValueName(const QString &name) const;
};

QDataStream &operator <<(QDataStream &s, const Config::Value &value);
QDataStream &operator >>(QDataStream &s, Config::Value &value);
QDataStream &operator <<(QDataStream &s, const Config::Mask &mask);
QDataStream &operator >>(QDataStream &s, Config::Mask &mask);
QDataStream &operator <<(QDataStream &s, const Config::Word &word);
QDataStream &operator >>(QDataStream &s, Config::Word &word);
QDataStream &operator <<(QDataStream &s, const Config &config);
QDataStream &operator >>(QDataStream &s, Config &config);

} //namespace

#endif
