/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "c18_compile.h"

#include "c18_config.h"
#include "c18.h"
#include "devices/list/device_list.h"

//-----------------------------------------------------------------------------
QStringList C18::CompileFile::genericArguments(const Compile::Config &config) const
{
  QStringList args;
  args += config.includeDirs(Tool::Compiler, "-I");
  args += "-I" + config.directory(Compile::HeaderDirectory).path(false);
  if( _data.device!=Device::AUTO_DATA.name ) {
    args += "-p";
    args += "%DEVICE";
  }
  args += config.customOptions(Tool::Compiler);
  args += "%I";
  return args;
}

Compile::FileData::List C18::CompileFile::files() const
{
  Compile::FileData::List files;
  files += fileData(PURL::Object, Compile::Generated);
  return files;
}

void C18::CompileFile::logStderrLine(const QString &line)
{
  // #### TODO
  _manager->log(Log::Data, line);
}

//-----------------------------------------------------------------------------
QStringList C18::Link::genericArguments(const Compile::Config &config) const
{
  QStringList args;
  args += "/k%LKR_PATH";
  args += "%LKR_NAME";
  args += "/l" + config.directory(Compile::LibraryDirectory).path(false);
  args += config.customOptions(Tool::Linker);
  args += "/o%COFF";
  args += "/m%MAP";
  args += "%OBJS";
  args += "%LIBS";
  args += config.customLibraries(Tool::Linker);
  return args;
}

Compile::FileData::List C18::Link::files() const
{
  Compile::FileData::List files;
  PURL::Url lkr = Main::toolGroup().linkerScript(_data.project, _data.linkType);
  files += Compile::FileData(lkr, Compile::Included | Compile::InProject);
  files += fileData(PURL::Map, Compile::Generated | Compile::InProject);
  files += fileData(PURL::Hex, Compile::Generated | Compile::InProject);
  if (_data.project) files += fileData(PURL::Coff, Compile::Generated | Compile::InProject);
  else files += fileData(PURL::Coff, Compile::Generated);
  files += fileData(PURL::Lst, Compile::Generated | Compile::InProject);
  files += fileData(PURL::Cod, Compile::Generated);
  return files;
}

void C18::Link::logStderrLine(const QString &line)
{
  // #### TODO
  _manager->log(Log::Data, line);
}
